#INDEX\
proc iwidgets::Optionmenu {} {}
#
# Optionmenu
# ----------------------------------------------------------------------
# Implements an option menu widget with options to manage it. 
# An option menu displays a frame containing a label and a button.
# A pop-up menu will allow for the value of the button to change. 
#
# ----------------------------------------------------------------------
#  AUTHOR:  Alfredo Jahn             Phone: (214) 519-3545
#                                    Email: ajahn@spd.dsccc.com
#                                           alfredo@wn.com
#
#  @(#) $Id: optionmenu.itk,v 1.1 1996/01/02 15:17:43 mulferts Exp $
# ----------------------------------------------------------------------
#            Copyright (c) 1995 DSC Technologies Corporation
# ======================================================================
# Permission to use, copy, modify, distribute and license this software 
# and its documentation for any purpose, and without fee or written 
# agreement with DSC, is hereby granted, provided that the above copyright 
# notice appears in all copies and that both the copyright notice and 
# warranty disclaimer below appear in supporting documentation, and that 
# the names of DSC Technologies Corporation or DSC Communications 
# Corporation not be used in advertising or publicity pertaining to the 
# software without specific, written prior permission.
# 
# DSC DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING 
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, AND NON-
# INFRINGEMENT. THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, AND THE
# AUTHORS AND DISTRIBUTORS HAVE NO OBLIGATION TO PROVIDE MAINTENANCE, 
# SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. IN NO EVENT SHALL 
# DSC BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR 
# ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS ACTION,
# ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS 
# SOFTWARE.
# ======================================================================

#
# Default resources.
#
option add *Optionmenu.borderWidth 2 widgetDefault
option add *Optionmenu.highlightThickness 2 widgetDefault
option add *Optionmenu.relief raised widgetDefault
option add *Optionmenu.labelPos w  widgetDefault
option add *Optionmenu.labelMargin 2 widgetDefault
option add *Optionmenu.popupCursor arrow widgetDefault

#
# Usual options.
#
tk++::usual Optionmenu {
    keep -activebackground -activeborderwidth -activeforeground \
	 -background -borderwidth -cursor -disabledforeground -font \
	 -foreground -highlightcolor -highlightthickness -labelfont \
	 -popupcursor
}

# ------------------------------------------------------------------
#                            OPTONMENU
# ------------------------------------------------------------------
class iwidgets::Optionmenu {
    inherit iwidgets::Labeledwidget
    
    constructor {args} {}
    destructor {}

    itk_option define -clicktime clickTime ClickTime 150
    itk_option define -command command Command {}
    itk_option define -cyclicon cyclicOn CyclicOn true
    itk_option define -width width Width 0
    itk_option define -font font Font -Adobe-Helvetica-Bold-R-Normal--*-120-*
    itk_option define -items items Items ""
    itk_option define -state state State normal
    itk_option define -highlightthickness highlightThickness \
	    HighlightThickness 2
    itk_option define -borderwidth borderWidth BorderWidth 2

    public method index {index} 
    public method delete {first last} 
    public method disable {index} 
    public method enable {args} 
    public method get {} 
    public method insert {index string args} 
    public method select {index} 
    public method sort {mode} 

    private method _getNextItem {index} 
    private method _next {} 
    private method _postMenu {} 
    private method _previous {} 
    private method _setItem {item} 
    private method _setSize {when} 
    
    private variable _currentItem ""
    protected variable _calcSize ""  ;# non-null => _calcSize pending
}

#
# Provide a lowercased access method for the Optionmenu class.
# 
proc ::iwidgets::optionmenu {pathName args} {
    uplevel ::iwidgets::Optionmenu $pathName $args
}

# ------------------------------------------------------------------
#                        CONSTRUCTOR
# ------------------------------------------------------------------
body iwidgets::Optionmenu::constructor {args} {
    component hull configure -borderwidth 0 -highlightthickness 0
    
    #
    # Create the menu frame that houses the gizmo and selected option menu
    # item text.
    #
    itk_component add menuFrame {
		frame $itk_interior.menuFrame 
    } {
		keep -background -cursor -relief -highlightcolor
		rename -highlightbackground -background background Background
    }
    pack $itk_component(menuFrame) -side left -fill x -expand yes
    pack propagate $itk_component(menuFrame) no
    
    #
    # Create the gizmo.  This is the small icon to the right of the selected
    # option menu item text.
    #
    itk_component add menuBtn {
		frame $itk_component(menuFrame).menuBtn \
			-width 12 -height 6
    } {
		keep -background -cursor -relief -borderwidth
    }
    pack $itk_component(menuBtn) -side right -padx 10
    
    #
    # Create the label part of the option menu that displays the selected
    # option menu item text.
    #
    itk_component add menuLabel {
		label $itk_component(menuFrame).menuLabel
    } {
		keep -background -cursor -font -foreground
    }
    pack $itk_component(menuLabel) -padx 1 -pady 2 \
	    -fill x -fill y -anchor c
    
    #
    # Create the popup menu. This menu does not get packed since its a 
    # popup that gets displayed when the user clickes on the menu button area.
    #
    itk_component add popupMenu {
		menu $itk_interior.popupMenu -tearoff no
    } {
		keep -activebackground -activeforeground -activeborderwidth \
			-background -borderwidth -disabledforeground \
			-font -foreground

		rename -cursor -popupcursor popupCursor Cursor
    }
    
    #
    # Create a temp frame/label which is used to calculate font size.
    #
    frame $itk_component(menuFrame).f
    label $itk_component(menuFrame).f.l -text "Font Test" 
    pack $itk_component(menuFrame).f.l
    
    #
    # Explicitly handle configs that may have been ignored earlier.
    #
    eval itk_initialize $args
}

# ------------------------------------------------------------------
#                           DESTRUCTOR
# ------------------------------------------------------------------
body iwidgets::Optionmenu::destructor {} {
    if {$v(_calcSize) != ""} {after cancel $v(_calcSize)}
}

# ------------------------------------------------------------------
#                             OPTIONS
# ------------------------------------------------------------------

# ------------------------------------------------------------------
# OPTION: -highlightthickness
#
# Specifies the thickness of the highlight ring.
# ------------------------------------------------------------------
configbody iwidgets::Optionmenu::highlightthickness {
    $itk_component(menuFrame) configure \
	    -highlightthickness $itk_option(-highlightthickness)

    _setSize
}

# ------------------------------------------------------------------
# OPTION: -borderwidth
#
# Specifies the thickness of the relief border.
# ------------------------------------------------------------------
configbody iwidgets::Optionmenu::borderwidth {
    $itk_component(menuFrame) configure -borderwidth $itk_option(-borderwidth)

    _setSize
}

# ------------------------------------------------------------------
# OPTION -command
#
# Specifies a command to be evaluated upon change in option menu.
# ------------------------------------------------------------------
configbody iwidgets::Optionmenu::command {}

# ------------------------------------------------------------------
# OPTION -cyclicon
#
# Turns on/off the 3rd mouse button capability. This feature
# allows the right mouse button to cycle through the popup 
# menu list without poping it up. <shift>M3 cycles through
# the menu in reverse order.
# ------------------------------------------------------------------
configbody iwidgets::Optionmenu::cyclicon {
    if {$itk_option(-cyclicon)} {
    	#
    	# Bind the 3rd mouse button.
    	#
    	bind $itk_component(menuFrame) <3> [code $this _next]
    	bind $itk_component(menuBtn)   <3> [code $this _next]
    	bind $itk_component(menuLabel) <3> [code $this _next]
    	
    	bind $itk_component(menuFrame) <Shift-3> [code $this _previous]
    	bind $itk_component(menuBtn)   <Shift-3> [code $this _previous]
    	bind $itk_component(menuLabel) <Shift-3> [code $this _previous]
    } else {
    	#
    	# Un-Bind 3rd mouse button.
    	#
    	bind $itk_component(menuFrame) <3> ""
    	bind $itk_component(menuBtn)   <3> ""
    	bind $itk_component(menuLabel) <3> ""
    	
    	bind $itk_component(menuFrame) <Shift-3> ""
    	bind $itk_component(menuBtn)   <Shift-3> ""
    	bind $itk_component(menuLabel) <Shift-3> ""
    }
}

# ------------------------------------------------------------------
# OPTION -width
#
# Allows the menu label width to be set to a fixed size
# ------------------------------------------------------------------
configbody iwidgets::Optionmenu::width {
    _setSize
}

# ------------------------------------------------------------------
# OPTION -font
#
# Change all fonts for this widget. Also re-calculate height based
# on font size (used to line up menu items over menu button label).
# ------------------------------------------------------------------
configbody iwidgets::Optionmenu::font {
    $itk_component(menuFrame).f.l configure -font $itk_option(-font)
    
    _setSize
}

# ------------------------------------------------------------------
# OPTION -items
#
# Create a list of items available on the menu. Used to create the
# popup menu.
# ------------------------------------------------------------------
configbody iwidgets::Optionmenu::items {
	if {$itk_option(-items) != ""} {
		set savedCurrentItem $_(_currentItem)
		
		#
		# Delete the old menu.
		#
		$itk_component(popupMenu) delete 0 last
		foreach opt $itk_option(-items) {
			$itk_component(popupMenu) add command -label " $opt " -command [code $this _setItem $opt]
		}
		set first [lindex $itk_option(-items) 0]
		
		#
		# Make sure "savedCurrentItem" is still in the list.
		#
		if {$first != ""} {
			if {[lsearch -exact $itk_option(-items) $savedCurrentItem] != -1} {
			select $savedCurrentItem
			} else {
			select $first
			}
		} else {
			$itk_component(menuLabel) configure -text ""
		}
		
		
		_setSize

		#
		# Delete old menu and clear the menu button label.
		#
	} else {
		$itk_component(popupMenu) delete 0 last
		_setItem ""
	}
}

# ------------------------------------------------------------------
# OPTION -state
#
# Specified one of two states for the Optionmenu: normal, or
# disabled.  If the Optionmenu is disabled, then option menu
# selection is ignored.
# ------------------------------------------------------------------
configbody iwidgets::Optionmenu::state {
    switch $itk_option(-state) {
    	normal {
    	    bind $itk_component(menuFrame) <1> [code $this _postMenu]
    	    bind $itk_component(menuBtn)   <1> [code $this _postMenu]
    	    bind $itk_component(menuLabel) <1> [code $this _postMenu]
    	} 
    	disabled {
    	    bind $itk_component(menuFrame) <1> {}
    	    bind $itk_component(menuBtn)   <1> {}
    	    bind $itk_component(menuLabel) <1> {}
    	} 
    	default {
    	    error "bad state option \"$itk_option(-state)\":\
		    should be disabled or normal"
    	}
    }
}

# ------------------------------------------------------------------
#                            METHODS
# ------------------------------------------------------------------

# ------------------------------------------------------------------
# METHOD: index index
#
# Return the numerical index corresponding to index.
# ------------------------------------------------------------------
body iwidgets::Optionmenu::index {index} {
    if {[regexp {(^[0-9]+$)} $index]} {
		return [$itk_component(popupMenu) index $index]
	
    } elseif {$index == "end"} {
		return [llength $itk_option(-items)]
	
    } elseif {$index == "select"} {
		return [lsearch $itk_option(-items) $_(_currentItem)]
	
    } else {
		set numValue [lsearch -glob $itk_option(-items) $index]
	
		if {$numValue == -1} {
			error "bad Optionmenu index \"$index\""
		} else {
			return $numValue
		}
    }
}

# ------------------------------------------------------------------
# METHOD: delete first ?last?
#
# Remove an item (or range of items) from the popup menu. 
# ------------------------------------------------------------------
body iwidgets::Optionmenu::delete {first {last {}}} {
    set first [index $first]
    
    if {$last != {}} {
		set last [index $last]
    } else {
		set last $first
    }
    
    set nextAvail $_(_currentItem)
    
    #
    # If current item is in delete range point to next available.
    #
    if {([llength $itk_option(-items)] > 1) && \
	    ([lsearch -exact [lrange $itk_option(-items) $first $last] \
 	     [get]] != -1)} {
		set nextAvail [_getNextItem $last]
    }
    
    configure -items [lreplace $itk_option(-items) $first $last]
    
    #
    # Make sure "nextAvail" is still in the list.
    #
    if {[lsearch -exact $itk_option(-items) $nextAvail] != -1} {
		_setItem $nextAvail
    } else {
		_setItem ""
    }
}

# ------------------------------------------------------------------
# METHOD: disable index
#
# Disable a menu item in the option menu.  This will prevent the user
# from being able to select this item from the menu.  This only effects
# the state of the item in the menu, in other words, should the item
# be the currently selected item, the user is responsible for 
# determining this condition and taking appropriate action.
# ------------------------------------------------------------------
body iwidgets::Optionmenu::disable {index} {
    set index [index $index]
    
    $itk_component(popupMenu) entryconfigure $index -state disabled
}

# ------------------------------------------------------------------
# METHOD: enable index
#
# Enable a menu item in the option menu.  This will allow the user
# to select this item from the menu.  
# ------------------------------------------------------------------
body iwidgets::Optionmenu::enable {index} {
    set index [index $index]
    
    $itk_component(popupMenu) entryconfigure $index -state normal
}

# ------------------------------------------------------------------
# METHOD: get
#
# Returns the current menu item.
# ------------------------------------------------------------------
body iwidgets::Optionmenu::get {} {
    return $_(_currentItem)	
}

# ------------------------------------------------------------------
# METHOD: insert index string ?string?
#
# Insert an item in the popup menu.
# ------------------------------------------------------------------
body iwidgets::Optionmenu::insert {index string args} {
    set index [index $index]
    set args [linsert $args 0 $string]
    
    configure -items [eval linsert {$itk_option(-items)} $index $args]
}

# ------------------------------------------------------------------
# METHOD: select index
#
# Select an item from the popup menu to display on the menu label
# button. 
# ------------------------------------------------------------------
body iwidgets::Optionmenu::select {index} {
    set index [index $index]
    
    _setItem [lindex $itk_option(-items) $index]
}

# ------------------------------------------------------------------
# METHOD: sort mode
#
# Sort the current menu in either "ascending" or "descending" order.
# ------------------------------------------------------------------
body iwidgets::Optionmenu::sort {mode} {
    switch $mode {
	ascending -
	increasing {
	    configure -items [lsort -increasing $itk_option(-items)]
	}
	descending -
	decreasing {
	    configure -items [lsort -decreasing $itk_option(-items)]
	}
	default {
	    error "bad sort argument \"$mode\": should be ascending,\
		    descending, increasing, or decreasing"
	}
    }
}
# ------------------------------------------------------------------
# PRIVATE METHOD: _getNextItem index
#
# Allows either a string or index number to be passed in, and returns
# the next item in the list in string format. Wrap around is automatic.
# ------------------------------------------------------------------
body iwidgets::Optionmenu::_getNextItem {index} {
    set index [expr $index + 1]
    
    if {$index >= [llength $itk_option(-items)]} {
	set index 0   ;# wrap around
    }
    return [lindex $itk_option(-items) $index]
}

# ------------------------------------------------------------------
# PRIVATE METHOD: _next
#
# Sets the current option label to next item in list if that item is
# not disbaled.
# ------------------------------------------------------------------
body iwidgets::Optionmenu::_next {} {
    if {$itk_option(-state) == "normal"} {
	set i [lsearch -exact $itk_option(-items) $_(_currentItem)]
	
	for {set cnt 0} {$cnt < [llength $itk_option(-items)]} {incr cnt} {
	    set i [expr $i + 1]
	    
	    if {$i >= [llength $itk_option(-items)]} {
		set i 0
	    }
	    
	    if {[lindex [$itk_component(popupMenu) \
		    entryconfigure $i -state] 4] != "disabled"} {
		_setItem [lindex $itk_option(-items) $i]
		break
	    }
	}
    }
}

# ------------------------------------------------------------------
# PRIVATE METHOD: _previous
#
# Sets the current option label to previous item in list if that 
# item is not disbaled.
# ------------------------------------------------------------------
body iwidgets::Optionmenu::_previous {} {
    if {$itk_option(-state) == "normal"} {
	set i [lsearch -exact $itk_option(-items) $_(_currentItem)]
	
	for {set cnt 0} {$cnt < [llength $itk_option(-items)]} {incr cnt} {
	    set i [expr $i - 1]
	    
	    if {$i < 0} {
		set i [expr [llength $itk_option(-items)] - 1]
	    }
	    
	    if {[lindex [$itk_component(popupMenu) \
		    entryconfigure $i -state] 4] != "disabled"} {
		_setItem [lindex $itk_option(-items) $i]
		break
	    }
	}
    }
}

# ------------------------------------------------------------------
# PRIVATE METHOD: _postMenu time
#
# Display the popup menu. Menu position is calculated.
# ------------------------------------------------------------------
body iwidgets::Optionmenu::_postMenu {} {
    #
    # Don't bother to post if menu is empty.
    #
    if {$itk_option(-items) == {}} {
		return
    }
    
    #
    # Calculate the position to post the popup.
    #
    set itemIndex [lsearch -exact $itk_option(-items) \
	    $_(_currentItem)]
	#
	# Activate the currentItem, so the menu comes up active
	#
    $itk_component(popupMenu) activate $itemIndex
	    
    set x [expr [winfo rootx $itk_component(menuFrame)] + \
	    $itk_option(-borderwidth) + $itk_option(-highlightthickness)]
    set y [expr [winfo rooty $itk_component(menuFrame)] - \
	    [$itk_component(popupMenu) yposition $itemIndex] + \
	    $itk_option(-borderwidth) + $itk_option(-highlightthickness)]
    
    #
    # Post the popup and do a global grab.
    #
	tk_popup $itk_component(popupMenu) $x $y
}

# ------------------------------------------------------------------
# PRIVATE METHOD: _setItem
#
# Set the menu button label to item, then dismiss the popup menu.
# Also check if item has been changed. If so, also call user-supplied
# command.
# ------------------------------------------------------------------
body iwidgets::Optionmenu::_setItem {item} {
    if {$_(_currentItem) != $item} {
		set _(_currentItem) $item
		$itk_component(menuLabel) configure -text $item
		
		if {[winfo ismapped $itk_component(hull)]} {
			uplevel #0 $itk_option(-command)
		}
    }
}

# ------------------------------------------------------------------
# PRIVATE METHOD: _setSize ?when?
#
# Set the size of the option menu.  If "when" is "now", the change 
# is applied immediately.  If it is "later" or it is not specified, 
# then the change is applied later, when the application is idle.
# ------------------------------------------------------------------
body iwidgets::Optionmenu::_setSize {{when later}} {
    if {$when == "later"} {
		if {$v(_calcSize) == ""} {
			set v(_calcSize) [after idle [code $this _setSize now]]
		}
		return
    } elseif {$when != "now"} {
		error "bad option \"$when\": should be now or later"
    }

    set v(_calcSize) ""

    #
    # Update idletasks and then configure the width and height of the
    # optionmenu button taking into account the borderwidth and
    # highlightthickness.  If the width has a value other than zero
    # than set it explicity.
    #
    update idletasks

    $itk_component(menuFrame) configure \
	    -height [expr [winfo reqheight $itk_component(menuFrame).f] + \
	    2 * $itk_option(-borderwidth) + \
	    2 * $itk_option(-highlightthickness)]

    if {$itk_option(-width)} {
    	$itk_component(menuFrame) configure -width $itk_option(-width)
    } else {
	$itk_component(menuFrame) configure -width \
	    [expr [winfo reqwidth $itk_component(popupMenu)] + \
	    2 * $itk_option(-borderwidth) + \
	    2 * $itk_option(-highlightthickness) + \
	    [winfo reqwidth $itk_component(menuBtn)] + 20]
    }
}


